#ifndef __MAINMENU_H__
#define __MAINMENU_H__

#include "../SexyAppFramework/Widget.h"
#include "../SexyAppFramework/ButtonListener.h"
#include "CurvedVal.h"

namespace Sexy
{

class WinDMApp;
class PodButton;
class MemoryImage;
class HyperlinkWidget;
class SoundInstance;
class ImageFont;

#define NUM_CENTER_TEXTS 4

class MainMenu : public Widget, public ButtonListener
{
public:
	enum
	{
		STATE_SHOWING,
		STATE_SHOWN,
		STATE_HIDING,
		STATE_HIDDEN,
		STATE_PRE_LAUNCH,
		STATE_LAUNCHING
	};

public:
	WinDMApp*				mApp;
	bool					mNoBackground;	
	MemoryImage*			mBackground;
	MemoryImage*			mTopFontGlowImage;
	int						mStartCountdown;

	bool					mLaunched;	

	float					mCenterTextShow[NUM_CENTER_TEXTS];
	float					mCenterWaviness[NUM_CENTER_TEXTS];

	PodButton*				mActionButton;
	PodButton*				mClassicButton;
	PodButton*				mPuzzleButton;
	PodButton*				mEndlessButton;
	
	PodButton*				mSecretButton;
	bool					mHasASecretModeUnlocked;

	PodButton*				mOptionsButton;
	PodButton*				mScoresButton;
	PodButton*				mMoreGamesButton;
	PodButton*				mQuitButton;
	HyperlinkWidget*		mChangeLink;	

	int						mMoveDelay;
	CurvedVal				mButtonsOffset;
	CurvedVal				mMainOffset;

	int						mModeSelected;	
	
	int						mState;	
	int						mLastRotButton;
	int						mRotButtonCount;

	int						mShutdownTimer;
	float					mMainLightFactor;
	float					mLightFactors[3];	
	float					mLightNum;
	float					mLaunchVolume;
	float					mCenterLightPct;
	SoundInstance*			mLaunchSoundInstance;
	SoundInstance*			mOldLaunchSoundInstance;

	int						mCreateUserDelay;
	int						mWelcomeDialogDelay;
	
	ImageFont*				mBaseShrunkNameFont;
	ImageFont*				mShrunkNameFont;

protected:
	void					SelectMode(int theMode);
	void					StartGame();		
	void					DrawTopText(Graphics* g);
	void					CreateFontGlowImages();
	void					DrawMouseover(Graphics* g, int theDestX, int theDestY, int theHintNum, float theShowPct, float theWaviness);
	void					RehupGameButtons();
	void					ResizeComponents();

public:
	MainMenu(WinDMApp* theApp);
	virtual ~MainMenu();

	void					Show();
	void					Hide();

	void					StartShutdownTimer();
	void					CancelNewGame();

	void					SyncProfile();
	void					SetBackground(MemoryImage* theImage);

	virtual void			AddedToManager(WidgetManager* theWidgetManager);
	virtual void			RemovedFromManager(WidgetManager* theWidgetManager);	

	virtual void			Update();
	virtual void			Draw(Graphics* g);

	virtual void			MouseDown(int x, int y, int theClickCount);
	virtual void			ButtonPress(int theId);
	virtual void			ButtonDepress(int theId);	
};

	
}

#endif //__MAINMENU_H__